/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package dk.nsp.api.jira

import dk.nsp.models.FeedbackEntity
import dk.nsp.shared.ApiCallCallFailed
import dk.nsp.shared.ApiCallCallSuccess
import dk.nsp.shared.Failed
import dk.nsp.shared.Success
import dk.nsp.testassertions.assertEquals
import dk.nsp.testassertions.assertIs
import dk.nsp.testassertions.assertIsAndApply
import io.mockk.coEvery
import io.mockk.mockk
import kotlinx.coroutines.runBlocking
import org.junit.jupiter.api.Test

class JiraControllerImplTest {

    class SendRating {
        private val failingApi = mockk<JiraApi>() {
            coEvery {
                sendRating(any())
            } returns ApiCallCallFailed(Exception("Could not send rating"))
        }

        private val api = mockk<JiraApi>() {
            coEvery {
                sendRating(any())
            } returns ApiCallCallSuccess(Unit)
        }

        @Test
        fun success() = runBlocking{
            JiraControllerImpl(api)
                .sendRating(this, FeedbackEntity(""))
                .await()
                .assertIs<Success<Unit>>()
        }

        @Test
        fun failed() = runBlocking {
            JiraControllerImpl(failingApi)
                .sendRating(
                    this, FeedbackEntity(
                        ""
                    )
                ).await()
                .assertIsAndApply<Failed> {
                    error.message.assertEquals("Could not send rating")
                }
        }
    }
}