/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package dk.nsp.domain.repository

import android.content.Context
import dk.nsp.api.IBackendApiLayer
import dk.nsp.api.jira.IJiraController
import dk.nsp.models.FeedbackEntity
import dk.nsp.shared.Failed
import dk.nsp.shared.Success
import dk.nsp.shared.toSuccess
import dk.nsp.testassertions.assertIs
import io.mockk.coEvery
import io.mockk.every
import io.mockk.mockk
import io.mockk.mockkObject
import kotlinx.coroutines.runBlocking
import org.junit.jupiter.api.Test

class FeedbackRepositoryTest {

    private val mockedController = mockk<IJiraController> {
        coEvery {
            sendRating(any(), any()).await()
        } returns Unit.toSuccess()

    }

    private val failingController = mockk<IJiraController> {
        coEvery {
            sendRating(any(), any()).await()
        } returns Failed(Exception())

    }

    private val mockBackend = mockk<IBackendApiLayer> {
        every {
            jiraController
        } returns mockedController
    }

    private val failingBackend = mockk<IBackendApiLayer> {
        every {
            jiraController
        } returns failingController
    }

    private fun getRepo(): FeedbackRepository {
        mockkObject(IBackendApiLayer)
        every {
            IBackendApiLayer.getLayer(any())
        } returns mockBackend

        val mockedContext = mockk<Context>(relaxed = true)

        return FeedbackRepository(mockedContext)
    }

    private fun getFailingRepo(): FeedbackRepository {
        mockkObject(IBackendApiLayer)
        every {
            IBackendApiLayer.getLayer(any())
        } returns failingBackend

        val mockedContext = mockk<Context>(relaxed = true)

        return FeedbackRepository(mockedContext)
    }

    @Test
    fun sendRating() = runBlocking {
        getRepo().sendRating(this, FeedbackEntity(""))
            .await()
            .assertIs<Success<Unit>>()

        getFailingRepo().sendRating(this, FeedbackEntity(""))
            .await()
            .assertIs<Failed>()
    }
}